    /////////////////////////////////////////////////////////////////////////
// Programme Tracker pour avion RC 
/////////////////////////////////////////////////////////////////////////
//30/05/2017 V1.0

//*************************************************************************
// Designed by Bernard Vermillet http://aeroplan.raidghost.com/
//*************************************************************************
 

#pragma config FOSC = INTOSCIO  //  Internal CLK (RA4 RA5 en I/O))
#pragma config WDTE =  OFF      //  Watch dog disabled
#pragma config PWRTE = OFF      //  Power-up timer disabled
#pragma config MCLRE = OFF      //  MCLR disable -  Port enabled
#pragma config CP = OFF         //  Code protect disables
#pragma config BOREN = ON       //  Brown-out reset selection bit enabled
#pragma config IESO = OFF       //  Switch over disable
#pragma config FCMEN = OFF      //  Fail safe clock disable

//include <pic16f644.h>
#include <htc.h>


#define sleep()     asm("SLEEP")
#define nop()       asm("NOP")

#define     ON      1   
#define     OFF     0
#define     true    1
#define     false   0   

#define     LED         PORTCbits.RC4 
#define     POWER_ACC   PORTCbits.RC3
#define     SWITCH_IN   PORTAbits.RA5
#define     BUZZER      PORTCbits.RC5

// Constantes 
// Nombre de Bip 
#define BEEP_ALARM                  3     // Bip...Bip...Bip..............Bip...Bip...BIP
#define BEEP_WAKE_UP                3     // Bip...Bip... Bip 1 fois au rveil
#define BEEP_SLEEP                  2     // Bip...Bip 1 fois  l'arrt du module
#define BEEP_LOW_BATTERY            100   // Bip........ Bip.......Bip  si batterie faible

// Toutes les dure sont sur une base d'units de 10ms  

// Attention si char dclar Max 255 sinon il faut dclarer un int

// Constantes diode LED
#define     LED_TIME_ON             5      // Dure d'activation de la LED (100ms)
#define     LED_TIME_OFF            95     // Dure d'extinction de la LED (1000ms)

#define     LED_TIME_ON_ALARM       30      // Dure d'activation si l'alarme est passe  ON     
#define     LED_TIME_OFF_ALARM      20      // Dure d'exctinction si l'alarme est passe  ON
                                            // Ceci permet de voir si l'alarme s'est dclenche
// Constantes pour le buzzer
#define     BEEP_TIME_AL_ON         30      // Dure du Bip du buzzer en alarme et wake_up
#define     BEEP_TIME_AL_OFF        15      // Dure entre Bip du buzzer en alarme et wake_up

#define     BEEP_SET_UP_DONE        150     // Longueur du bip aprs talonnage 

#define     SALVE_INTERVAL          50      // En mode alarme accelromtre, interval entre les salves.

#define     BEEP_TIME_SL_ON         10      // Dure du Bip du buzzer en mode sleep 
#define     BEEP_TIME_SL_OFF        30      // Dure entre Bip du buzzer en mode sleep 

#define     BEEP_TIME_BATT_ON       5       // Dure du du buzzer en mode batterie faible
#define     BEEP_TIME_BATT_OFF      100     // Dure du du buzzer en mode batterie faible

// Constantes pour l'alarme acclration units 10ms (attention la variable doit tre un int)
#define     ACCEL_ALARM_DELAY      6000     // Dure d'attente avant alarme 
#define     FIRST_DELAY            6000     // Delai initial avant de grer l'alarme acclration 

// Constante pour valider l'talonnage de mesure de tension batterie
#define     TIME_SW_LOW             600     // Dure basse du switch pour entrer en mode talonnage

// Au cas ou un talonnage serait fait avec une batterie trop basse 
// En considrant la mesaure  +/- 5% cela donne U batt min = 3.56 
#define     TH_MIN                  728     //Seuil mini d'talonnage

//******************************************************************************

// Le convertisseur donne 1024 pour 2.5V (valeur de la refrence de tension)
// L'acclromtre a une sensibilit de 300mV/G soit une valeur AD de 122/g
// le seuil est fix  0.5g soit 0.5 x 122 
#define     ACCEL_TRESHOLD      61     // Seuil de dtection d'acclration  


// Constantes pour l'entr filtrage du switch poussoir
#define     FILTER_TIME            25  // filtre de l'entre INT units 

// Constantes pour le convertisseur AD
// Right justify, Ref Ext , A/D converter module is operating
#define X       0b11011001  // AD 6
#define Y       0b11010101  // AD 5
#define Z       0b11010001  // AD 4
#define U_BATT  0b11001101  // AD 3

// Prototype des fonction
//***********************
// General
void wait(unsigned int) ;           // Temps d'attente en units 10ms
void Init() ;


// Commande de la LED (Flash))
void Flash_Led();                   // active le flash de la  led 

// activation du buzzer (Dure du Bip, Dure de repos, Nb de Bip) 
void Active_Buzzer(unsigned char,unsigned char,unsigned char); 

// Mode d'alarme
void Test_Accel() ;     // Tester les acclrations 
void Battery_Test() ;   // Test de la batterie 
void Set_Up_Battery();  // Etalonnage mesure


// Variables Globales
unsigned int  Count_IT ;                // Compteur IT pour fonctions wait

struct
    {
    unsigned    Run_Prog            :1; // Demande d'exuter le programme
    unsigned    Accel               :1;	// 1 si acclration dtecte
    unsigned    Accel_Fail          :1; // Pas d'acclration
    unsigned    Salve_Done          :1; // 1 salve termine
    unsigned    Buzzer_Busy         :1; // Le buzzer est occup
    unsigned    Battery_Alarm       :1; // Alarma batterie basse
    unsigned    Wake_Up             :1; // Demande de rveille
    unsigned    Filter_Done         :1; // La durdu filtre est atteinte
    unsigned    Batt_Me_Enable      :1; // Mesure de tension batterie autorise
    unsigned    Beep_Wake_Up_Done   :1; // Bip wake up termins
    unsigned    Wait_Flag           :1; // Utilis par la fonction Wait()
    unsigned    Set_Up_Batt         :1; // Demande de set up batterie 
    unsigned    Set_Up_Batt_Done    :1; // Demande effectue
    unsigned    Alarm_Memory        :1; // Passe  1 Si il y a eut alarme
    unsigned    IT_SWITCH_LOW       :1; // Une IT sur front ngatif du  switch a eut lieu 
    unsigned    Init                :1; // Permet d'initialiser des variables
     
    } STATUS_Sbits ;
    
    

//***************************************************************************
void interrupt isr() 

{
            
    if (INTCONbits.T0IF == true)     // une IT timer0 toutes les 10ms
    {
        INTCONbits.T0IF = 0 ;
        // Overflow timer0
        TMR0 = 99  ;         // Re-load (256-99) soit  157 x 64  x  = 10ms
        
        Count_IT ++ ;
        
        STATUS_Sbits.Run_Prog =true ;
    }
    
       
    if (INTCONbits.RAIF == true & INTCONbits.RAIE == true)
    {
        
        if (SWITCH_IN == 0 || PCONbits.nPOR == 0 )
        {
            PCONbits.nPOR = 1;
            //Suspendre l'IT sur RA5 jusqu'  l'arrt de la pression sur le switch + filtrage
            INTCONbits.RAIE = 0 ; 
            INTCONbits.RAIF = 0 ; 
            STATUS_Sbits.Wake_Up = !STATUS_Sbits.Wake_Up ;
            STATUS_Sbits.Beep_Wake_Up_Done = false ;  
            STATUS_Sbits.Salve_Done = false ;
            STATUS_Sbits.Accel = true ;
            STATUS_Sbits.Alarm_Memory = false ;
            STATUS_Sbits.Accel_Fail = false ;
            STATUS_Sbits.Set_Up_Batt_Done = false ;
            STATUS_Sbits.IT_SWITCH_LOW = true ;  
            STATUS_Sbits.Buzzer_Busy = false;
            STATUS_Sbits.Init = true ;
        }
        else 
        {
            INTCONbits.RAIF = 0 ;
        }        
    }

}

//****************************************************************************
                          // Initialisations  //
//****************************************************************************
void main()
{
            Init() ;
}
//****************************************************************************
void Init()
{
    static unsigned char Count_High = 0;
    static unsigned int  Count_Low ;
  
    
    //----------------
    // Initialisations
    //----------------
    
    // OSCCON option oscillateur 1MHZ  (pour faible consommation soit ~100uA)
    OSCCON = 0b01000000;
  
    // OPTION_REG  pull-up valides, INT front ngatif, prescaler Timer0 = 16 
    // CLK timer = CLK / 4  soit une horloge timer de 64s
    OPTION_REG = 0b00000011;
    
     
    
    TRISC = 0b00000111 ;
    // Port   0 = OUT   1 = IN
    // RC0,RC1,RC2 entre AD Z,Y,X
    // RC3 Sortie Power ACC
    // RC4 Sortie Cde de diode LED
    // RC5 sortie Buzzer
    
    
    TRISA = 0b00111111;
    // RA0 NU
    // RA1 Entre mesure tension de rfrence   
    // RA2 NU
    // RA3 NU
    // RA4 Entre AD de mesure de la tension batterie 
    // RA5 Entre Switch
   
    WPUAbits.WPUA5 = 1 ; // Pull-up sur RA5 (entre switch)
        
    // Convertisseur AD et registre ANSEL 0 = digital I/O   1 = AD input 
    // AN1 rfrence    AN3 mesure Ubatt     AN4,AN5,AN6 entre acclromtre
    ANSEL =  0b01111010 ;
        
    // Interruptions
    INTCON = 0b11101000 ;       
    // Global interrupt enabled -Peripheral interrupt disable -
    // Timer0 interrupt enabled - External int disable
    // PortA on-change INT enable
    
    IOCAbits.IOCA5 = 1 ;    // IT sur RA5  ON CHANGE (entre switch))
    
    BUZZER = OFF ;
    LED = OFF ;
    POWER_ACC = ON ;      // Power ACC ON
    
    STATUS_Sbits.Salve_Done = false ;
    STATUS_Sbits.Wake_Up = true ;
    STATUS_Sbits.Battery_Alarm = false  ;  // A priori
    STATUS_Sbits.Batt_Me_Enable = false ;
        

           
//*****************************************************************************
                         // Programme principal //
//*****************************************************************************
    
    Main_loop :
          
    if (STATUS_Sbits.Run_Prog == true) 

    {       
        STATUS_Sbits.Run_Prog = false ;
            
        // Pour filtrer les rebond ne revalider l'IT et effectuer le mode que si 
        // le switch a t relach depuis un certain temps (FILTER_TIME)
        if (SWITCH_IN == 0 || STATUS_Sbits.IT_SWITCH_LOW == true)
        {
         Count_High = 0 ;
         STATUS_Sbits.Filter_Done = false ;
         STATUS_Sbits.IT_SWITCH_LOW = false ;
        }
        if (SWITCH_IN == 1)
        {
            Count_High ++ ;
            if (Count_High > FILTER_TIME)
            {
                INTCONbits.RAIF = 0 ; // clear it pending
                INTCONbits.RAIE = 1 ; 
                STATUS_Sbits.Filter_Done = true ;
            }       
        }
      
         
        // Au rveil du module sur IT INT
        //-------------------------------
        if (STATUS_Sbits.Wake_Up == true &  STATUS_Sbits.Beep_Wake_Up_Done == false & STATUS_Sbits.Set_Up_Batt == false)      
        {              
            POWER_ACC = ON ;      // Power ACC
            if (STATUS_Sbits.Beep_Wake_Up_Done == false & STATUS_Sbits.Salve_Done == false)
            {
                Active_Buzzer(BEEP_TIME_AL_ON, BEEP_TIME_AL_OFF, BEEP_WAKE_UP) ;
            }
            else if (STATUS_Sbits.Salve_Done == true)
            {
                STATUS_Sbits.Beep_Wake_Up_Done = true ; 
            
            }
           
        }

   
        // A la mise en veille sur IT INT 
        //-------------------------------
        if (STATUS_Sbits.Wake_Up == false & STATUS_Sbits.Filter_Done == true)  // Demande de mise en sommeil
         {    
            if (STATUS_Sbits.Salve_Done== false)
            {
                Active_Buzzer(BEEP_TIME_SL_ON, BEEP_TIME_SL_OFF, BEEP_SLEEP) ;
            }
            else    
            {
                BUZZER = OFF ;
                LED = OFF ;
                POWER_ACC = OFF ;    
                sleep();   
            }    
        }
        
               
        Flash_Led (); 
        
        Test_Accel();   

              
        // Tester l'tat bas du bouton et Si > TIME_SW_LOW
        // En vue de l'talonnage batterie
        if (SWITCH_IN == 1)
        { 
            Count_Low = 0 ;
        }
        else if (STATUS_Sbits.Set_Up_Batt_Done == false)
        {
            Count_Low ++ ;
            if (Count_Low > TIME_SW_LOW) 
            {
                // Effectuer le Set up batterie juste une fois au rveil
                BUZZER = OFF ;
                Set_Up_Battery() ;
                STATUS_Sbits.Set_Up_Batt = false ; 
                STATUS_Sbits.Set_Up_Batt_Done = true ; 
            }   
            
        }
    
        // Ne pas effectuer le test batterie si pas d'acclration accel 
        // l'alarme accelration est prioritaire et tant que le rveil n'est pas
        // complet bips et tempo comprise       
        if (STATUS_Sbits.Accel_Fail == false & STATUS_Sbits.Beep_Wake_Up_Done == true) 
        {                               
            Battery_Test() ; 
        }
    }
    goto Main_loop ;       
}
//*****************************************************************************
                    // Fin du programme principal //
//*****************************************************************************

//*****************************************************************************
                         // Sous Programme  //
//*****************************************************************************
void Test_Accel() 
{
    // Dtection des acclrations 
    //----------------------------
    static unsigned int         X_AD_Result ;   
    static unsigned long        X_Average_R;    // Moyenne rapide
    static unsigned long        X_Average_L;    // Moyenne lente
    
    static unsigned int         Y_AD_Result ;
    static unsigned long        Y_Average_R;
    static unsigned long        Y_Average_L;
    
    static unsigned int         Z_AD_Result ;
    static unsigned long        Z_Average_R;
    static unsigned long        Z_Average_L;
    
    static unsigned int         Alarm_Count ;
    static unsigned char        Count_interval = 0 ; 
    
    static unsigned int  First_Delay ;
    
           

    // La moyenne est effectue sans multiplication ni division
    // Pour une plus grande prcision les valeur instantanes sont multiplie par 64
    // Le seuil est donc  multipli par le lmme facteur
    #define ACCEL_TRESHOLD_AMP = ACCEL_TRESHOLD * 64
    unsigned int Seuil  ACCEL_TRESHOLD_AMP ; 
    
    // Par ailleurs il est ncessaire de filtrer les valeurs instantanes
    // Ce filtre s'effectue sous forme de moyenne glissante sur 32 chantillons
    // de la forme Average_R = ((64 x Average_R - Average_R) + Instantan) / 64
   
    // La moyenne glissante servant de base aux comparaisons est tablie sur 256 chantillons 
    // sous la forme Average_L  = ((256 x Average_L - Average_L) + Instantan) / 256
        
    
    // Traitement des donnes de l'acclromtre
    //******************************************    
    STATUS_Sbits.Accel = false ;    //  priori 
    if (STATUS_Sbits.Init ==  true)   
    {
        Alarm_Count = 0 ;
        STATUS_Sbits.Init = false ;
        X_AD_Result = 0; 
        Y_AD_Result = 0; 
        Z_AD_Result = 0; 
    }
     
    if (First_Delay < FIRST_DELAY)
    {   
        First_Delay ++ ;
        return ;
    }   
 
    //Conversion voix X
    //-----------------
    ADCON0 = X ;  
    for (int count =0; count < 1 ; count++); // attente environ 40s 
     ADCON0bits.GO_DONE=1 ;                  // start sample
    //pulling attente de fin de conversion ( i.e  GO_DONE_bit =0)
    while (ADCON0bits.GO_DONE);     // Boucler tant que le bit est  1
    X_AD_Result   = ADRESL << 6;    // Rsultat x 64
    X_AD_Result+  = ADRESH << 14 ;
    
    //Moyenne rapide sur 64 chantillons
    X_Average_R = (((X_Average_R << 6)  - X_Average_R) + X_AD_Result)>> 6 ;
    
    // Moyenne glissante sur 256 chantillons
    X_Average_L = (((X_Average_L << 8)  - X_Average_L) + X_Average_R)>> 8 ;
   
    //Comparaison au seuil          
    if (X_Average_R > X_Average_L + Seuil || X_Average_R < X_Average_L - Seuil )
    {
        STATUS_Sbits.Accel = true ;    //Seuils atteints 
    }
        
    //Conversion voix Y
    //-----------------
    ADCON0 = Y ;           
    for (int count =0; count < 1 ; count++); // attente environ 40s 
    ADCON0bits.GO_DONE=1 ;              // start sample
    //pulling attente de fin de conversion ( i.e  GO_DONE_bit =0)
    while (ADCON0bits.GO_DONE);     // Boucler tant que le bit est  1
    Y_AD_Result   = ADRESL << 6;    // Rsultat x 64
    Y_AD_Result+  = ADRESH << 14 ;
    
    //Moyenne rapide sur 64 chantillons
    Y_Average_R = (((Y_Average_R << 6)  - Y_Average_R) + Y_AD_Result)>> 6 ;
        
    // Moyenne glissante sur 256 chantillons
    Y_Average_L = (((Y_Average_L << 8) - Y_Average_L) + Y_AD_Result)>> 8;
        
    //Comparaison au seuil      
    if (Y_Average_R > Y_Average_L + Seuil || Y_Average_R < Y_Average_L - Seuil )
    {
        STATUS_Sbits.Accel = true ;    //Seuils atteints 
    }     
        
    //Conversion voix Z
    //-----------------
    ADCON0 = Z ;           
    for (int count =0; count < 1 ; count++); // attente environ 40s 
    ADCON0bits.GO_DONE=1 ;              // start sample
    //pulling attente de fin de conversion ( i.e  GO_DONE_bit =0)
    while (ADCON0bits.GO_DONE);     // Boucler tant que le bit est  1
    Z_AD_Result   = ADRESL << 6;    // Rsultat x 64
    Z_AD_Result+  = ADRESH << 14 ;
    
    //Moyenne rapide sur 64 chantillons
    Z_Average_R = (((Z_Average_R << 6)  - Z_Average_R) + Z_AD_Result)>> 6 ;
        
    // Moyenne glissante sur 256 chantillons
    Z_Average_L = (((Z_Average_L << 8) - Z_Average_L ) + Z_AD_Result)>> 8 ;
        
    //Comparaison au seuil        
    if (Z_Average_R > Z_Average_L + Seuil || Z_Average_R < Z_Average_L - Seuil )
    {
        STATUS_Sbits.Accel = true ;    //Seuils atteints 
    }             
    
    //Bilan sur la dttection d'acclration
    //---------------------------------------
    if (STATUS_Sbits.Accel == false)
    {
        // Dlai avant alarme
        Alarm_Count ++ ;
        if (Alarm_Count > ACCEL_ALARM_DELAY) 
        {
              STATUS_Sbits.Accel_Fail = true ;  
        }
    }
    else  
    {
        Alarm_Count = 0 ;
        STATUS_Sbits.Accel_Fail = false ;
    }
    
    if(STATUS_Sbits.Salve_Done == false &  STATUS_Sbits.Accel_Fail == true )  
    {
        STATUS_Sbits.Alarm_Memory = true ;
        Active_Buzzer(BEEP_TIME_AL_ON, BEEP_TIME_AL_OFF, BEEP_ALARM) ;
    }
    
    if (STATUS_Sbits.Salve_Done== true)
    {
        Count_interval ++  ; 
    
        if (Count_interval > SALVE_INTERVAL)  
        {
            STATUS_Sbits.Salve_Done = false ;  // recommencer une salve
            Count_interval = 0;
        }   
    }

} 
//****************************************************************************
void Battery_Test()
{
     
    // adresses de EEPROM
    //  $0  Set up effectu, le contenu sera $AA
    //  $1  LB du seuil calcul
    //  $2  HB du seuil calcul
    //  $3  Copie LB du seuil calcul
    //  $4  Copie HB du seuil calcul
    
    unsigned int    U_Batt ;
    unsigned int    U_Batt_threshold ;
    unsigned char   HB1 , LB1 , HB2 , LB2 , ID ;
                
    STATUS_Sbits.Battery_Alarm = false ;  // a priori
               
    // Tester si l'talonnage est effectu (Smaphore en flash)  et si il est intgre
    ID  = eeprom_read (0) ;
    HB1 = eeprom_read (1) ;
    LB1 = eeprom_read (2) ;
    HB2 = eeprom_read (3) ;
    LB2 = eeprom_read (4) ;
    
    if (ID!= 0xAA || HB1 != HB2 || LB1 != LB2 ) 
    {
        // Sinon Alarm batterie
        if (STATUS_Sbits.Buzzer_Busy ==  false)
        {
            STATUS_Sbits.Salve_Done = false ;
        }
        STATUS_Sbits.Battery_Alarm = true ;
        STATUS_Sbits.Set_Up_Batt = true ;
        
        if (STATUS_Sbits.Salve_Done == false) 
        {
             Active_Buzzer(BEEP_TIME_BATT_ON, BEEP_TIME_BATT_OFF, BEEP_LOW_BATTERY) ;
        }
    }
    else
    {
        // Si test OK alors mesure de la tension
        ADCON0 = U_BATT ;           
        for (int count =0; count < 1 ; count++); // attente environ 120s 
        ADCON0bits.GO_DONE=1 ;              // start sample
        //pulling attente de fin de conversion ( i.e  GO_DONE_bit =0)
        while (ADCON0bits.GO_DONE);     // Boucler tant que le bit est  1
      
        U_Batt   = ADRESL ;    
        U_Batt+  = ADRESH << 8 ;
     
        // Lecture de la valeur du seuil (HB Adresse 1 , LB Adresse  2  de la flash)
        // Si tension basse Alarm batterie 
        U_Batt_threshold    = LB1 ;
        U_Batt_threshold+   = HB1 << 8;
                
        
        if ( U_Batt < U_Batt_threshold )
        {
            if (STATUS_Sbits.Buzzer_Busy == false)
            {
                STATUS_Sbits.Salve_Done = false ;
            }
            STATUS_Sbits.Battery_Alarm = true ;
            if (STATUS_Sbits.Salve_Done == false) 
            {
                Active_Buzzer(BEEP_TIME_BATT_ON, BEEP_TIME_BATT_OFF, BEEP_LOW_BATTERY) ; 

            }  
        }  
        else
        {
            STATUS_Sbits.Salve_Done == false ; // Si il tait en alarme le stopper
            BUZZER = OFF ;
        }
    }  
}
     
//*****************************************************************************
// Etalonnage de la mesure de la tension batterie et mise en mmoire flash
 void Set_Up_Battery()
 {
    unsigned int    U_Batt ; 
    unsigned int    Threshold ;
    
    // Mesure de la tension batterie
   
    ADCON0 = U_BATT ; 
    for (int count =0; count < 1 ; count++); // attente environ 120s 
    ADCON0bits.GO_DONE=1 ;              // start sample
    //pulling attente de fin de conversion ( i.e  GO_DONE_bit =0)
    while (ADCON0bits.GO_DONE);     // Boucler tant que le bit est  1
    
    U_Batt   = ADRESL ;    
    U_Batt+  = ADRESH << 8 ;

    // Le convertisseur donne 1023 pour 2.5V (valeur de la refrence de tension)
    // 30% de charge de la batterie corespondant  0.8928 la valeur pleine charge
    // Une multiplication par un nombre non entier prendrait plus de 500 octets
    // on prendra donc une valeur multiple binaire comme 0.8928 x 64 soit 57
    // et on gardera que les octets hauts cette solution prend qu'une centaine d'octets
    
    Threshold = ((U_Batt * 57) >> 6) ;  
    
    if (Threshold < TH_MIN)  
    {
        STATUS_Sbits.Wake_Up = true ;       // Pour viter qu'il se rendorme car une INT a eut lieu
        wait(100) ;
        return ;        //  Lors de l'talonnage la batterie est mauvaise ou pas charge
    }
       
    unsigned char LB = Threshold ;
    unsigned char HB = Threshold >> 8 ;
     
    eeprom_write(0, 0xAA);  // Signifit que l'talonnage  t effectu
    eeprom_write(1, HB) ;  
    eeprom_write(2, LB) ;
    eeprom_write(3, HB) ;
    eeprom_write(4, LB) ; 
    
    BUZZER = ON ;
    LED = ON; 
    
    wait(BEEP_SET_UP_DONE) ;
    BUZZER = OFF ;
    LED = OFF ;
    
    STATUS_Sbits.Set_Up_Batt = false ;  // Fin de demande de set_up
    STATUS_Sbits.Wake_Up = true ;       // Pour viter qu'il se rendorme car une INT a eut lieu
    STATUS_Sbits.Salve_Done = true ;    // Pour viter les bips d'veil
    wait(100) ;
    
 }

//*****************************************************************************
//Flash LED
void Flash_Led()
{
    static unsigned char Count_ON ;
    static unsigned char Count_OFF;
    unsigned     char    Time_ON ;
    unsigned     char    Time_OFF;
      
    Time_OFF = LED_TIME_OFF ;
    Time_ON  = LED_TIME_ON ;
    
    if (STATUS_Sbits.Alarm_Memory == true)
     { 
        Time_ON  = LED_TIME_ON_ALARM ;
        Time_OFF = LED_TIME_OFF_ALARM ;
         
     }
    
    //Si Allume
    if (LED == true & Count_ON < Time_ON)
    {
        Count_ON ++ ;
    }
    else
    { 
        LED = OFF;
        Count_ON = 0;
    }
    
    //Si Eteinte
    if (LED == false & Count_OFF < Time_OFF )
    {
        Count_OFF ++ ;
    }
    else 
    {
        LED = ON;
        Count_OFF = 0;
    }
}

//*****************************************************************************
//Activation du buzzer 
void Active_Buzzer(unsigned char On_Time, unsigned char Off_Time, unsigned char Nb_Beep)
{
    static unsigned char Count_ON ;
    static unsigned char Count_OFF ;
    static unsigned char Count_Beep ;
    
    if (SWITCH_IN == 0)
    {
        BUZZER = OFF  ;
        return ;
    }
    
    if (STATUS_Sbits.Buzzer_Busy == false)
    {
        Count_Beep = 0 ;
        BUZZER = ON ;   // Commencer par Buzzer ON 
        STATUS_Sbits.Buzzer_Busy = true ;
    }
          
    if (STATUS_Sbits.Salve_Done == false & Count_Beep < Nb_Beep)
    {
        //Si buzzer actif
        if (BUZZER == ON & Count_ON < On_Time)
        {
            BUZZER = ON ;
            Count_ON ++ ;
        }
        else 
        {
            BUZZER = OFF;
            Count_ON = 0;        
        }
    
        //Si buzzer inactif
        if (BUZZER == OFF    & Count_OFF < Off_Time)
        {
            BUZZER = OFF ;
            Count_OFF ++ ;
        }
        else 
        {
            BUZZER = ON ;
            Count_OFF = 0;            
        }
    }
    else if(STATUS_Sbits.Salve_Done == true || Count_Beep >= Nb_Beep || SWITCH_IN ==0)
    {
        {
        Count_Beep = 0;
        Count_ON = 0 ;
        Count_OFF = 0;
        STATUS_Sbits.Salve_Done = true ; // salve effectue
        STATUS_Sbits.Buzzer_Busy = false ;
        BUZZER = OFF ;
        }
    }
    
    if (Count_ON == On_Time-1) 
    {
        Count_Beep ++ ;
    }

}
    
//*****************************************************************************
void wait(unsigned int Delay)
{
    
    int Count1 = Count_IT + Delay ;
    while(Count_IT < Count1)  ;  

}
//***************************************************************************** 
   
 